
# q-expansion of newform 10000.2.a.c, downloaded from the LMFDB on 01 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 10000
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [-1, 0], [0, 0], [0, 1], [4, -2], [0, -3], [-4, 2], [1, 3], [-5, -2], [-3, 1], [3, 0], [1, 2], [-2, -2], [3, -3], [0, 1], [-1, 4], [9, -3], [5, -6], [-6, -2], [6, -1], [-9, 0], [5, -5], [5, -2], [-4, 8], [-2, 3], [-1, -4], [-3, 9], [9, -12], [10, 0], [15, -3], [-10, 16], [0, 11], [-7, 8], [-5, 0], [1, 8], [-9, 9], [7, -10], [-11, 0], [-9, 9], [9, -16], [3, 4], [4, 6], [8, 10], [4, -6], [0, -6], [12, -9], [3, -10], [-6, -10], [-16, -2], [18, -6], [2, 8], [23, 4], [5, 4], [23, -10], [21, -3], [14, 5], [-14, -2], [8, 0], [-21, 6], [7, -5], [28, -3], [-22, -4], [-8, 2], [2, 17], [-18, -2], [-1, -14], [12, -18], [6, -3], [23, 5], [18, -6], [16, 5], [-2, 19], [20, -9], [-19, 15], [-1, 22], [-21, 20], [15, 0], [21, -13], [13, -22], [9, 12], [7, -4], [32, 0], [15, -24], [25, -3], [28, -21], [-25, 8], [9, 7], [-14, 12], [7, 10], [13, -18], [-12, 25], [9, -3], [29, -12], [21, -36], [-2, 9], [30, -12], [-16, 7], [17, -20], [15, 3], [16, -18], [25, 6], [6, 2], [-8, -1], [23, 4], [-3, -18], [-14, 32], [-9, -6], [-14, -5], [7, -4], [12, 15], [30, -9], [-2, 21], [13, -2], [-33, -4], [9, -2], [2, 5], [16, -24], [18, -30], [16, -5], [-18, -4], [3, -27], [6, -10], [10, -1], [11, -4], [-27, 15], [-20, 34], [-15, 30], [27, 6], [-19, 9], [-7, 21], [-3, 21], [-16, 20], [15, 16], [23, -12], [-3, 25], [6, -12], [-17, 31], [18, 10], [11, 2], [18, -21], [11, -6], [-24, 27], [-44, 6], [9, 13], [-34, 3], [-9, 3], [2, 28], [3, -27], [9, 12], [27, 9], [-18, 30], [-28, 20], [28, 12], [-20, 16], [11, -36], [39, -2], [39, -23], [20, -35], [27, 15], [-39, 12], [6, 14], [4, 19], [-14, -16], [-5, 1], [-11, -14], [-9, 1], [3, -20], [15, -16], [-15, -20], [36, -35], [1, 28], [15, -31], [3, 25], [-21, -6], [7, 6], [-3, -34], [-10, 6], [45, -6], [15, -42], [-32, 29], [7, 32], [-12, -20], [5, -3], [22, 20], [19, -20], [23, -26], [33, -12], [12, -21], [-40, 10], [33, 15], [-26, -21], [-12, -8], [26, -21], [-14, 32], [22, -53], [2, -7], [-12, 8], [-3, -17], [18, 28], [57, -6], [14, -28], [-43, 42], [44, -24], [-39, 43], [18, -6], [25, -11], [6, -27], [12, -51], [3, -6], [-20, 36], [-37, 3], [31, -8], [27, -36], [-6, 3], [28, -41], [9, -24], [12, 7], [-22, -7], [-60, 6], [39, -36], [-19, -18], [20, 0], [-13, 26], [-31, 55], [-12, 40], [11, 33], [-33, 18], [5, -10], [-26, 3], [-38, 17], [-3, -37], [-24, 28], [-26, 18], [58, -2], [0, -12], [16, 14], [-13, -9], [-7, -19], [-40, 5], [13, -20], [21, -9], [24, -22], [-43, 19], [-36, -3], [16, -35], [48, -41], [15, -24], [-11, 38], [-23, 6], [-32, 22], [15, -36], [-13, 20], [-9, 49], [23, 9], [-18, 18], [-39, 38], [-32, 23], [-12, -30], [24, -34], [-51, 26], [4, 30], [35, -19], [-26, -3], [3, 46], [10, 34], [18, -16], [16, -47], [-42, 48], [37, -6], [5, -38], [1, 2], [-38, 47], [65, -3], [-11, -18], [-24, 7], [-49, 16], [29, -32], [12, -34], [-65, 24], [-35, 11], [2, -1], [-20, 51], [12, 27], [27, -30], [52, -3], [-22, 20], [-11, 44], [11, 2], [35, -30], [13, -61], [-22, 8], [-4, -46], [-41, -26], [-45, 6], [16, -15], [27, -54], [23, 25], [13, 11], [33, -21], [-59, 14], [12, -17], [33, 18], [-31, 42], [-15, -12], [-59, 9], [-5, 14], [-9, -1], [-46, -3], [25, -40], [-15, -23], [48, -28], [2, 26], [-74, 2], [-40, -2], [-34, 19], [-39, 38], [31, -62], [-20, 41], [-23, 8], [13, 4], [-10, -29], [-10, -21], [-31, 26], [-65, 38], [-25, 2], [-24, 42], [-63, 21], [-65, 18], [-13, 17], [38, -4], [37, -15], [16, 7], [-45, -30], [-4, -4], [-36, -7], [17, 47], [43, -36], [-13, -47], [23, 18], [68, 0], [9, -46], [-47, -2], [42, 16], [-9, -1], [16, -25], [-6, 2], [0, -36], [15, -24], [-40, 76], [-21, 31], [27, -43], [9, -54], [-3, 30], [4, -60], [-25, -15], [-24, 35], [22, 1], [42, -3], [19, -64], [-36, 15], [58, -27], [-20, 4], [1, 19], [-48, 26], [-23, 47], [21, 45], [66, -3], [67, 9], [2, 15], [6, 54], [-48, 51], [5, 8], [-44, 13], [42, -13], [48, -12], [-30, 0], [-32, 50], [33, 26], [31, -62], [-34, 22], [-8, 40], [18, 21], [-42, -30], [51, -33], [60, -10], [67, -41], [-19, -21], [73, -12], [-6, 35], [20, -34], [-30, 45], [-33, 23], [8, 19], [60, -9], [17, 22], [-71, 44], [-18, 6], [-56, 2], [-21, -42], [-3, -35], [-21, -18], [-60, 22], [-60, 41], [50, 29], [-35, 35], [-15, -24], [-27, 43], [-16, 56], [-30, -2], [43, -46], [7, -12], [-8, -25], [8, 37], [-12, 0], [-71, 51], [-36, -34], [-1, -13], [-5, 11], [45, -46], [-3, -16], [-34, 13], [-49, 57], [61, -41], [42, 11], [-34, 60], [42, -45], [-36, 5], [57, -34], [-36, -22], [58, -26]]
